!
! $Id: analytical.F,v 1.12 2005/10/26 15:18:52 pmarches Exp $
!
#include "cppdefs.h"
!
!  ANALYTICAL PACKAGE:
!--------------------------------------------------------------------
!
!  This package is used to provide various analytical fields to the
!  model when appropriate.
!
!  Routines:
!
!  ana_bmflux_tile   Analytical kinematic bottom momentum flux.
!  ana_btflux_tile   Analytical kinematic bottom flux of tracer
!                          type variables.
!  ana_bsedim_tile   Analytical bottom sediment grain size
!                          and density.
!  ana_smflux_tile   Analytical kinematic surface momentum flux
!                          (wind stress).
!  ana_srflux_tile   Analytical kinematic surface shortwave
!                          radiation.
!  ana_ssh_tile      Analytical sea surface height climatology.      
!  ana_sst_tile      Analytical sea surface temperature and dQdSST  
!                         which are used during heat flux correction.
!  ana_sss_tile      Analytical sea surface salinity which is used 
!                         during salt flux correction.
!  ana_stflux_tile   Analytical kinematic surface flux of tracer type
!                          variables.
!  ana_tclima_tile   Analytical tracer climatology fields.     
!  ana_uclima_tile   Analytical tracer climatology fields.  
!  ana_wwave_tile    Analytical wind induced wave amplitude,
!                         direction and period.
!  ana_sediment_tile Analytical sediment
!  ana_psource_tile  Analytical point source
!  ana_bry_tile      Analytical boundary forcing.
!
!-------------------------------------------------------------------
!

# if !defined OPENMP
      integer function omp_get_thread_num()
      omp_get_thread_num=0
      return
      end
      integer function omp_get_num_threads()
      omp_get_num_threads=1
      return
      end
# endif

#ifdef ANA_BMFLUX
      subroutine ana_bmflux_tile (Istr,Iend,Jstr,Jend)
!
!---------------------------------------------------------------------
!  This routine sets kinematic bottom momentum flux (bottom stress)
! "bustr" and "bvstr" [m^2/s^2] using an analytical expression.
!---------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "forces.h"
# include "scalars.h"
      integer Istr,Iend,Jstr,Jend, i,j
!
# include "compute_auxiliary_bounds.h"
!
      do j=JstrR,JendR
        do i=Istr,IendR
          bustr(i,j)=???
        enddo
      enddo
      do j=Jstr,JendR
        do i=IstrR,IendR
          bvstr(i,j)=???
        enddo
      enddo
      return
      end
#endif /* ANA_BMFLUX */ 
#ifdef SOLVE3D
# ifdef ANA_BTFLUX || defined ANA_BSFLUX || defined ANA_BPFLUX 
      subroutine ana_btflux_tile (Istr,Iend,Jstr,Jend, itrc)
!
!---------------------------------------------------------------------
!  This routine sets kinematic bottom flux of tracer type variables
!  [tracer units m/s].
!
!  On Input:
!     itrc      Tracer type array index.
!---------------------------------------------------------------------
!
      implicit none
#  include "param.h"
#  include "grid.h"
#  include "forces.h"
#  include "scalars.h"
      integer itrc, Istr,Iend,Jstr,Jend, i,j
!
#  include "compute_auxiliary_bounds.h"
!
      if (itrc.eq.itemp) then
!
! Set kinematic bottom heat flux [degC m/s] at horizontal RHO-points.
!--------------------------------------------------------------------
!
#  if defined BASIN     || defined CANYON_A  || defined CANYON_B   \
  || defined GRAV_ADJ  || defined OVERFLOW  || defined SEAMOUNT   \
  || defined SHELFRONT || defined SED_TEST2 || defined UPWELLING  \
  || defined REGIONAL  || defined INNERSHELF || defined RIVER     \
  || defined VORTEX
        do j=JstrR,JendR
          do i=IstrR,IendR
            btflx(i,j,itemp)=0.
          enddo
        enddo
#  else
        do j=JstrR,JendR
          do i=IstrR,IendR
            btflx(i,j,itemp)=???
          enddo
        enddo
#  endif
#  ifdef SALINITY
      elseif (itrc.eq.isalt) then
!
!  Set kinematic bottom salt flux (m/s) at horizontal RHO-points,
!  scaling by bottom salinity is done in STEP3D.
!---------------------------------------------------------------------
!
#   if defined SHELFRONT || defined SED_TEST2  || defined UPWELLING  \
   || defined REGIONAL  || defined INNERSHELF || defined SEAMOUNT   \
   || defined RIVER     || defined VORTEX
        do j=JstrR,JendR
          do i=IstrR,IendR
            btflx(i,j,isalt)=0.
          enddo
        enddo
#   else
        do j=JstrR,JendR
          do i=IstrR,IendR
            btflx(i,j,isalt)=???
          enddo
        enddo
#   endif
#  endif /* SALINITY */
      else
!
!---------------------------------------------------------------------
!  Set kinematic surface flux of additional tracers, 
! for example sediments, bio..., to zero
!---------------------------------------------------------------------
!
        do j=JstrR,JendR
          do i=IstrR,IendR
            btflx(i,j,itrc)=0.
          enddo
        enddo
      endif
      return
      end
# endif /* ANA_BTFLUX */
#endif /* SOLVE3D */
!
#if defined ANA_BSEDIM && defined BBL
      subroutine ana_bsedim (tile)
      implicit none
# include "param.h"
      integer tile
# include "compute_tile_bounds.h"
      call ana_bsedim_tile   (Istr,Iend,Jstr,Jend)
      return
      end
      subroutine ana_bsedim_tile (Istr,Iend,Jstr,Jend)
!
!---------------------------------------------------------------------
!  This routine sets initial bottom sediment grain diameter size [m]
!  and density used in the bottom boundary formulation [kg/m^3].
!---------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "bbl.h"
# include "grid.h"
# include "scalars.h"
      integer Istr,Iend,Jstr,Jend, i,j
!
# include "compute_auxiliary_bounds.h"
!
# if defined SED_TEST2 || defined REGIONAL
!
! taucb=critical threshold stress for initiation of motion 
! (=bedload for coarse grains). 
! critical suspension stress: ustar_crit=0.8*w_set 
!
! determine taucb from Shields curve, fit provided by 
! Soulsby & Whitehouse 1997, Threshold of sediment motion 
! in coastal environments, Proc. Pacific Coasts and Ports 
! '97 Conf., pp 149--154, Univ Canterbury, Nw Zealand.
!
! visk=1.3e-3/rhow; (kinem. visc., nu=mu/rhow)
! D=d50*(g*(Sdens/rhow-1)/(visk^2))^0.33333 
! thetcr=0.3./(1+1.2*D) + 0.055*(1-exp(-0.02*D))
! taucb=thetcr.*(g*(sdens-rhow).*d50);
!
! Souslby's (1997) estimate of settling velocity
!   w_set = visk*(sqrt(10.36^2+1.049*D^3)-10.36)/d50 [m/s]
! with D as above 
!
      do j=JstrR,JendR
        do i=IstrR,IendR
          Ssize(i,j)=1.5e-4  ! d50 [m]
          Sdens(i,j)=2650.0  ! rho sediment [kg/m^3]
          taucb(i,j)=0.16    ! critical bedload stress [N/m^2]
          w_set(i,j)=0.013   ! analytical settling velocity [m/s]
          Hripple(i,j)=0.01   ! analytical initial ripple height [m]
          Lripple(i,j)=0.10   ! analytical initial ripple length [m]
        enddo
      enddo
# else
      ANA_BSEDIM: no values provided for SSIZE and SDENS.
# endif
      return
      end
#endif /* ANA_BSEDIM && BBL */

#ifdef ANA_SMFLUX
      subroutine ana_smflux_tile (Istr,Iend,Jstr,Jend)
!
!  Sets kinematic surface momentum flux (wind stress) "sustr" and "svstr"
!  [m^2/s^2] using an analytical expression.
!
# ifdef AGRIF
      use Agrif_UTIL
# endif
      implicit none
# include "param.h"
# include "grid.h"
# include "forces.h"
# include "scalars.h"
      integer Istr,Iend,Jstr,Jend, i,j
      real Ewind, Nwind, dircoef, windamp
      real cff1, cff2
      data windamp /0./ 
      data Ewind, Nwind, dircoef /0., 0., 0./
      save windamp
!
# include "compute_auxiliary_bounds.h"
!
!  Set kinematic surface momentum flux (wind stress) component in the
!  XI-direction (m^2/s^2) at horizontal U-points.
!
# ifdef BASIN
      cff1=0.0001 * 0.5*(1.+tanh((time-6.*86400.)/(3.*86400.)))
      cff2=2.*pi/el
      do j=JstrR,JendR
        do i=Istr,IendR
          sustr(i,j)=-cff1*cos(cff2*yr(i,j))
       enddo
      enddo
# elif defined CANYON_A || defined CANYON_B
      do j=JstrR,JendR
        do i=Istr,IendR
          sustr(i,j)=0.0001*0.5*sin(2.*pi*tdays/10.)*
     &               (1.-tanh((yr(i,j)-0.5*el)/10000.))
        enddo
      enddo
# elif defined SED_TEST2
      do j=JstrR,JendR
        do i=IstrR,IendR
          windamp=0.5+
     &            0.5*TANH((time-user(9))/user(10))
          sustr(i,j)=windamp*user(1)
        enddo
      enddo
# elif defined UPWELLING
      if (tdays.le.2.) then
        windamp=-0.1*sin(pi*tdays/4.)/rho0
      else
        windamp=-0.1/rho0
      endif
      do j=JstrR,JendR
        do i=Istr,IendR
          sustr(i,j)=windamp
        enddo
      enddo
# elif defined GRAV_ADJ  || defined OVERFLOW || defined SEAMOUNT \
    || defined SHELFRONT || defined SOLITON  || defined INNERSHELF \
    || defined REGIONAL  || defined RIVER    || defined VORTEX
      do j=JstrR,JendR
        do i=Istr,IendR
          sustr(i,j)=0.
        enddo
      enddo
# else
      do j=JstrR,JendR
        do i=Istr,IendR
          sustr(i,j)=???
        enddo
      enddo
# endif
!
!  Set kinematic surface momentum flux (wind stress) component in the
!  ETA-direction (m^2/s^2) at horizontal V-points.
!
# if defined BASIN     || defined CANYON_A  || defined CANYON_B  || \
     defined GRAV_ADJ  || defined OVERFLOW  || defined SEAMOUNT  || \
     defined SHELFRONT || defined SOLITON   || defined UPWELLING || \
     defined RIVER     || defined VORTEX
      do j=Jstr,JendR
        do i=IstrR,IendR
          svstr(i,j)=0.
        enddo
      enddo
# elif defined SED_TEST2
      do j=JstrR,JendR
        do i=IstrR,IendR
          windamp=0.5+
     &            0.5*TANH((time-user(9))/user(10))
          svstr(i,j)= windamp*user(2)
        enddo
      enddo
# elif defined INNERSHELF
      do j=Jstr,JendR
        do i=IstrR,IendR
          svstr(i,j)=-1.e-4
        enddo
      enddo
# else
      do j=Jstr,JendR
        do i=IstrR,IendR
          svstr(i,j)=???
        enddo
      enddo
# endif
      return
      end
#endif /* ANA_SMFLUX */
#ifdef ANA_SRFLUX
      subroutine ana_srflux_tile (Istr,Iend,Jstr,Jend)
!
!---------------------------------------------------------------------
!  This subroutine sets kinematic surface solar shortwave radiation
!  flux "srflx" (degC m/s) using an analytical expression.
!---------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "forces.h"
# include "scalars.h"
      integer Istr,Iend,Jstr,Jend, i,j
!
# include "compute_auxiliary_bounds.h"
!
!  Set kinematic surface solar shortwave radiation [degC m/s] at
!  horizontal RHO-points.
!
# if defined OVERFLOW   || defined REGIONAL || defined UPWELLING\
  || defined INNERSHELF || defined SEAMOUNT || defined SHELFRONT\
  || defined RIVER      || defined VORTEX
      do j=JstrR,JendR
        do i=IstrR,IendR
          srflx(i,j)=0.
        enddo
      enddo
# else
      do j=JstrR,JendR
        do i=IstrR,IendR
          srflx(i,j)=???
        enddo
      enddo
# endif
      return
      end
#endif /* ANA_SRFLUX */

#if defined ANA_SSH && defined ZCLIMATOLOGY
      subroutine ana_ssh (tile)
      implicit none
# include "param.h"
      integer tile
# include "compute_tile_bounds.h"
      call ana_ssh_tile (Istr,Iend,Jstr,Jend)
      return
      end 
!
      subroutine ana_ssh_tile (Istr,Iend,Jstr,Jend)
!
!---------------------------------------------------------------------
!  This routine sets analytical sea surface height climatology [m].
!---------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "climat.h"
# include "scalars.h"
      integer Istr,Iend,Jstr,Jend, i,j
!
# include "compute_auxiliary_bounds.h"
!
!  Set sea surface height (meters).
!
# if defined REGIONAL
      do j=JstrR,JendR
        do i=IstrR,IendR
          ssh(i,j)=0.
        enddo
      enddo
# else
      do j=JstrR,JendR
        do i=IstrR,IendR
          ssh(i,j)=???
        enddo
      enddo
# endif
      return
      end
#endif /* ANA_SSH && ZCLIMATOLOGY */

#if defined ANA_SST && defined QCORRECTION
      subroutine ana_sst_tile (Istr,Iend,Jstr,Jend)
!
!--------------------------------------------------------------------
!  This routine sets sea surface temperature SST[Celsius] and surface
!  net heat flux sensitivity dQdSTT to sea surface temperature using
!  analytical expressions. dQdSTT is usually computed in units of
!  [Watts/m^2/degC]. It needs to be scaled to [m/s] by dividing by
!  rho0*Cp.  These forcing fields are used when the heat flux
!  correction is activated:
!
!       Q_model ~ Q + dQdSST * (T_model - SST)
!--------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "forces.h"
# include "scalars.h"
      integer Istr,Iend,Jstr,Jend, i,j
!
# include "compute_auxiliary_bounds.h"
!
      do j=JstrR,JendR
        do i=IstrR,IendR
          sst(i,j)=???
          dqdt(i,j)=???
        enddo
      enddo
      return
      end
#endif /* ANA_SST && QCORRECTION */
#if defined SALINITY && defined QCORRECTION && \
    defined SFLX_CORR && defined ANA_SSS
      subroutine ana_sss_tile (Istr,Iend,Jstr,Jend)
!
!--------------------------------------------------------------------
!  This routine sets sea surface salinity SSS[PSU] using
!  analytical expressions. This forcing field is used when the
!  salt flux correction is activated:
!
!    SSSFLX_model ~ SSS*(E-P) + CST * (SSS_model - SSS)
!
!  we use DQDSST for CST.... 
!
!--------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "forces.h"
# include "scalars.h"
      integer Istr,Iend,Jstr,Jend, i,j
!
# include "compute_auxiliary_bounds.h"
!
      do j=JstrR,JendR
        do i=IstrR,IendR
          sss(i,j)=???
        enddo
      enddo
      return
      end
#endif /* SALINITY && QCORRECTION && SFLX_CORR && ANA_SSS */
#if defined ANA_STFLUX || defined ANA_SSFLUX
      subroutine ana_stflux_tile (Istr,Iend,Jstr,Jend, itrc)
!
!--------------------------------------------------------------------
!  This routine sets kinematic surface flux of tracer type variables
!  "stflx" (tracer units m/s) using analytical expressions.
!
!  On Input:
!     itrc      Tracer type array index.
!--------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "forces.h"
# include "scalars.h"
      integer itrc, Istr,Iend,Jstr,Jend, i,j 
c
# include "compute_auxiliary_bounds.h"
c
      if (itrc.eq.itemp) then
!
!  Set kinematic surface heat flux [degC m/s] at horizontal
!  RHO-points.
!
#if defined BASIN     || defined CANYON_A  || defined CANYON_B  ||\
    defined OVERFLOW  || defined GRAV_ADJ  || defined SHELFRONT ||\
    defined SEAMOUNT  || defined UPWELLING || defined REGIONAL  ||\
    defined INNERSHELF || defined RIVER    || defined VORTEX

        do j=JstrR,JendR
          do i=IstrR,IendR
            stflx(i,j,itemp)=0.
          enddo
        enddo
#else
        do j=JstrR,JendR
          do i=IstrR,IendR
            stflx(i,j,itemp)=???
          enddo
        enddo
#endif
#ifdef SALINITY
      elseif (itrc.eq.isalt) then
!
!  Set kinematic surface freshwater flux (m/s) at horizontal
!  RHO-points, scaling by surface salinity is done in STEP3D.
!
# if defined SHELFRONT  || defined UPWELLING || defined REGIONAL ||\
     defined INNERSHELF || defined SEAMOUNT  || defined RIVER
        do j=JstrR,JendR
          do i=IstrR,IendR
            stflx(i,j,isalt)=0.
          enddo
        enddo
# else
        do j=JstrR,JendR
          do i=IstrR,IendR
            stflx(i,j,isalt)=???
          enddo
        enddo
# endif
#endif /* SALINITY */
      else
!
!  Set kinematic surface flux of additional tracers, if any.
!
        do j=JstrR,JendR
          do i=IstrR,IendR
            stflx(i,j,itrc)=0.
          enddo
        enddo
      endif
      return
      end
#endif /* ANA_STFLUX || ANA_SSFLUX */
!---------------------------------------------------------------------
#if defined TCLIMATOLOGY
      subroutine ana_tclima (tile)
      implicit none
# include"param.h"
      integer tile
#include "compute_tile_bounds.h"
      call ana_tclima_tile   (Istr,Iend,Jstr,Jend)
      return
      end
      subroutine ana_tclima_tile (Istr,Iend,Jstr,Jend)
!
!---------------------------------------------------------------------
!  This routine sets analytical ACTIVE (T&S) tracer climatology fields.
!---------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "climat.h"
# include "ocean3d.h"
# include "scalars.h"
# include "sediment.h"
      integer Istr,Iend,Jstr,Jend, i,j,k
      real    cff,cff1
!
# include "compute_auxiliary_bounds.h"
!
!  Set climatology fields for tracer type variables.
!---------------------------------------------------------------------
!
# ifdef ANA_TCLIMA
      do k=1,N
        do j=JstrR,JendR
          do i=IstrR,IendR
            tclm(i,j,k,itemp)=???
#  ifdef SALINITY
            tclm(i,j,k,isalt)=???
#  endif /* SALINITY */
          enddo
        enddo
      enddo
# endif

# ifdef BIOLOGY
#  define temp cff
#  define SiO4 cff1
      do k=1,N
        do j=JstrR,JendR
          do i=IstrR,IendR
#  ifdef ANA_TCLIMA
            temp=t(i,j,k,1,itemp)
            if (temp.lt.8.) then
               SiO4=30.
            elseif (temp.ge.8. .and. temp.le.11.) then
               SiO4=30.-((temp-8.)*(20./3.))
            elseif (temp.gt.11. .and. temp.le.13.) then
                SiO4=10.-((temp-11.)*(8./2.))
            elseif (temp.gt.13. .and. temp.le.16.) then
               SiO4=2.-((temp-13.)*(2./3.))
            elseif (temp.gt.16.) then
              SiO4=0.
            endif
            tclm(i,j,k,iNO3_)=1.67+0.5873*SiO4+0.0144*SiO4**2
     &                            +0.0003099*SiO4**3
#   ifdef BIO_NChlPZD
            tclm(i,j,k,iChla)=0.08
            tclm(i,j,k,iPhy1)=0.1
            tclm(i,j,k,iZoo1)=0.06
            tclm(i,j,k,iDet1)=0.02
#   elif defined BIO_N2ChlPZD2
            tclm(i,j,k,iNH4_)=0.1
            tclm(i,j,k,iChla)=0.08
            tclm(i,j,k,iPhy1)=0.06
            tclm(i,j,k,iZoo1)=0.04
            tclm(i,j,k,iDet1)=0.02
            tclm(i,j,k,iDet2)=0.02
#   elif defined BIO_N2P2Z2D2
            tclm(i,j,k,iNH4_)=0.1
            tclm(i,j,k,iPhy1)=0.04
            tclm(i,j,k,iPhy2)=0.06
            tclm(i,j,k,iZoo1)=0.04
            tclm(i,j,k,iZoo2)=0.04
            tclm(i,j,k,iDet1)=0.02
            tclm(i,j,k,iDet2)=0.02
#   endif
#  else
            if (.not.got_tclm(iNO3_)) then
              temp=t(i,j,k,1,itemp)
              if (temp.lt.8.) then
                 SiO4=30.
              elseif (temp.ge.8. .and. temp.le.11.) then
                 SiO4=30.-((temp-8.)*(20./3.))
              elseif (temp.gt.11. .and. temp.le.13.) then
                 SiO4=10.-((temp-11.)*(8./2.))
              elseif (temp.gt.13. .and. temp.le.16.) then
                 SiO4=2.-((temp-13.)*(2./3.))
              elseif (temp.gt.16.) then
                SiO4=0.
              endif
              tclm(i,j,k,iNO3_)=1.67+0.5873*SiO4+0.0144*SiO4**2
     &                               +0.0003099*SiO4**3
            endif
#   ifdef BIO_NChlPZD
            if (.not.got_tclm(iChla)) tclm(i,j,k,iChla)=0.08
            if (.not.got_tclm(iPhy1)) tclm(i,j,k,iPhy1)=0.1
            if (.not.got_tclm(iZoo1)) tclm(i,j,k,iZoo1)=0.06
            if (.not.got_tclm(iDet1)) tclm(i,j,k,iDet1)=0.02
#   elif defined BIO_N2ChlPZD2
            if (.not.got_tclm(iNH4_)) tclm(i,j,k,iNH4_)=0.1
            if (.not.got_tclm(iChla)) tclm(i,j,k,iChla)=0.08
            if (.not.got_tclm(iPhy1)) tclm(i,j,k,iPhy1)=0.06
            if (.not.got_tclm(iZoo1)) tclm(i,j,k,iZoo1)=0.04
            if (.not.got_tclm(iDet1)) tclm(i,j,k,iDet1)=0.02
            if (.not.got_tclm(iDet2)) tclm(i,j,k,iDet2)=0.02
#   elif defined BIO_N2P2Z2D2
            if (.not.got_tclm(iNH4_)) tclm(i,j,k,iNH4_)=0.1
            if (.not.got_tclm(iPhy1)) tclm(i,j,k,iPhy1)=0.04
            if (.not.got_tclm(iPhy2)) tclm(i,j,k,iPhy2)=0.06
            if (.not.got_tclm(iZoo1)) tclm(i,j,k,iZoo1)=0.04
            if (.not.got_tclm(iZoo2)) tclm(i,j,k,iZoo2)=0.04
            if (.not.got_tclm(iDet1)) tclm(i,j,k,iDet1)=0.02
            if (.not.got_tclm(iDet2)) tclm(i,j,k,iDet2)=0.02
#   endif
#  endif /* ANA_TCLIMA */
          enddo
        enddo
      enddo
#  undef SiO4
#  undef temp
# endif /* BIOLOGY */

# ifdef SEDIMENT
      do k=1,N
        do j=JstrR,JendR
          do i=IstrR,IendR
#  ifdef ANA_TCLIMA
            tclm(i,j,k,isand)=Csed(1)
            tclm(i,j,k,isilt)=Csed(2)
#  else   
            if (.not.got_tclm(isand)) then
              tclm(i,j,k,isand)=0. !Csed(1)
            endif
            if (.not.got_tclm(isilt)) then
              tclm(i,j,k,isilt)=0. !Csed(2)
            endif
#  endif
          enddo
        enddo
      enddo
# endif /* SEDIMENT */

# ifdef PASSIVE_TRACER
      do k=1,N
        do j=JstrR,JendR
          do i=IstrR,IendR
#  ifdef ANA_TCLIMA
            tclm(i,j,k,itpas)=0.0
#  else
            if (.not.got_tclm(itpas)) then
              tclm(i,j,k,itpas)=0.0
            endif
#  endif
          enddo
        enddo
      enddo
# endif

      return
      end
#endif /* TCLIMATOLOGY */
!
!====================================================================
!                   subroutine ana_uclima
!====================================================================
!
#if defined ANA_M2CLIMA && defined M2CLIMATOLOGY ||\
   (defined ANA_M3CLIMA && defined M3CLIMATOLOGY)
      subroutine ana_uclima (tile)
      implicit none
# include "param.h"
      integer tile
# include "compute_tile_bounds.h"
      call ana_uclima_tile   (Istr,Iend,Jstr,Jend)
      return
      end
!---------------------------------------------------------------------
!
      subroutine ana_uclima_tile (Istr,Iend,Jstr,Jend)
!
!---------------------------------------------------------------------
!  This routine sets analytical momentum climatology fields.
!---------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "climat.h"
# include "scalars.h"
      integer Istr,Iend,Jstr,Jend, i,j,k
!
# include "compute_auxiliary_bounds.h"
!
# if defined ANA_M2CLIMA && defined M2CLIMATOLOGY
#  if defined REGIONAL
      do j=JstrR,JendR
        do i=IstrR,IendR
          ubclm(i,j)=0.
          vbclm(i,j)=0.
        enddo
      enddo
#  else
      do j=JstrR,JendR
        do i=IstrR,IendR
          ubclm(i,j)=???
          vbclm(i,j)=???
        enddo
      enddo
#   endif
# endif
# if defined ANA_M3CLIMA && defined M3CLIMATOLOGY && defined SOLVE3D
#  if defined REGIONAL
      do k=1,N
        do j=JstrR,JendR
          do i=IstrR,IendR
            uclm(i,j,k)=0.
            vclm(i,j,k)=0.
          enddo
        enddo
      enddo
#  else
      do k=1,N
        do j=JstrR,JendR
          do i=IstrR,IendR
            uclm(i,j,k)=???
            vclm(i,j,k)=???
          enddo
        enddo
      enddo
#  endif
# endif
      return
      end
#endif /* ANA_M2CLIMA && M2CLIMATOLOGY || (ANA_M3CLIMA && M3CLIMATOLOGY) */
!
!====================================================================
!                   subroutine ana_wwave
!====================================================================
!
#if defined ANA_WWAVE && defined BBL
      subroutine ana_wwave (tile)
      implicit none
# include "param.h"
      integer tile
# include "compute_tile_bounds.h"
      call ana_wwave_tile   (Istr,Iend,Jstr,Jend)
      return
      end
!
      subroutine ana_wwave_tile (Istr,Iend,Jstr,Jend)
!
!---------------------------------------------------------------------
!  This routine sets wind induced wave amplitude, direction
!  and period used in the bottom boundary layer formulation.
!---------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "forces.h"
# include "scalars.h"
      integer Istr,Iend,Jstr,Jend, i,j
!
# include "compute_auxiliary_bounds.h" 
!
!  Set wind induced wave amplitude (m), direction (radians) and
!  period (s) at RHO-points.
!
# if defined SED_TEST2 || defined REGIONAL
      do j=JstrR,JendR
        do i=IstrR,IendR
          Awave(i,j)=1.0
          Dwave(i,j)=270.*deg2rad
          Pwave(i,j)=10.
        enddo
      enddo
# else
      ANA_WWAVE: no values provided for AWAVE, DWAVE, and PWAVE.
       do j=JstrR,JendR
        do i=IstrR,IendR
          Awave(i,j)=???
          Dwave(i,j)=???
          Pwave(i,j)=???
        enddo
      enddo
# endif
      return
      end
#endif /* ANA_WWAVE && BBL */

#if defined SEDIMENT
      subroutine ana_sediment (tile)
      implicit none
# include "param.h"
      integer tile
# include "compute_tile_bounds.h"
      call ana_sediment_tile (Istr,Iend,Jstr,Jend)
      return
      end
!
      subroutine ana_sediment_tile (Istr,Iend,Jstr,Jend)
!
!---------------------------------------------------------------------
!  This routine sets sediment ripple and bed parameters
!---------------------------------------------------------------------
!
      implicit none
# include "param.h"
# include "grid.h"
# include "scalars.h"
# include "sediment.h"
# include "bbl.h"
      integer Istr,Iend,Jstr,Jend, i,j, ilay, itrc
!
#undef DEBUG 
#ifdef DEBUG
      integer ick,jck
      parameter (ick=60, jck=35)
#endif
# include "compute_auxiliary_bounds.h" 
# if defined SED_TEST2 || defined REGIONAL
      do j=JstrR,JendR
        do i=IstrR,IendR
#ifdef BBL 
# ifndef ANA_SEDIMENT 
            if(.not.got_inibed(1)) then 
# endif                    
             Hripple(i,j)=Hrip   ! initial ripple height [m] from .in file
# ifndef ANA_SEDIMENT 
            endif            
            if(.not.got_inibed(2)) then 
# endif                    
             Lripple(i,j)=Lrip   ! initial ripple length [m] from .in file
# ifndef ANA_SEDIMENT 
            endif            
# endif                    
#endif                    
          do ilay=1,NLAY
#ifndef ANA_SEDIMENT
            if(.not.got_inised(1)) then 
#endif                    
              bed_thick(i,j,ilay)=Bthk(ilay) 
#ifndef ANA_SEDIMENT
            endif            
            if(.not.got_inised(2)) then 
#endif                    
              bed_poros(i,j,ilay)=Bpor(ilay)
#ifndef ANA_SEDIMENT
            endif            
            if(.not.got_inised(3)) then 
#endif                    
             do itrc=1,NST
              bed_frac(i,j,ilay,itrc)=Bfr(ilay,itrc)
             enddo
#ifndef ANA_SEDIMENT
            endif            
#endif                    
          enddo

#ifdef DEBUG
         if(j.eq.jck.and.i.eq.ick) then
           write(6,*) '********** ANA_SEDIMENT ***********'
           do itrc=1,NST
             write(6,*) 'Sd(itrc)',Sd(itrc)
             do ilay=1,NLAY
               write(6,*) 'i,j,ilay,itrc',i,j,ilay,itrc
               write(6,*) 'bed_frac',bed_frac(i,j,ilay,itrc)
             enddo
           enddo
           do ilay=1,NLAY
             write(6,*) 'bed_thick',bed_thick(i,j,ilay)
             write(6,*) 'bed_por',bed_poros(i,j,ilay)
           enddo
         endif
#endif
        enddo
      enddo
# else
      ana_sediment: no values provided
       do j=JstrR,JendR
        do i=IstrR,IendR
          do ilay=1,NLAY
            bed_thick(i,j,ilay)=???
            bed_poros(i,j,ilay)=???
            do itrc=1,NST
              bed_frac(i,j,ilay,itrc)=???
            enddo
          enddo
        enddo
      enddo
# endif
      return
      end
#endif /* defined SEDIMENT */

#if defined PSOURCE && defined ANA_PSOURCE && defined SOLVE3D
!
!-----------------------------------------------------------
!  Set analytical tracer and mass point sources and sinks
!-----------------------------------------------------------
!
      subroutine ana_psource_tile (Istr,Iend,Jstr,Jend) 
      implicit none          
# include "param.h"
# include "scalars.h"
# include "sources.h"
# include "ocean3d.h"
# include "grid.h"
!
      integer is, k, Istr,Iend,Jstr,Jend, i,j
      real cff, cff1, cff2, ramp, Hs
# include "compute_auxiliary_bounds.h" 

      if (iic.eq.ntstart) then

!
! Set-up nondimensional shape Qshape, must add to unity
!
# if defined RIVER
#  define EXP_SHAPE
#  ifdef CST_SHAPE
       cff=1./float(N)
       do k=1,N                         ! Uniform vertical
         do is=1,Nsrc                   ! distribution
           Qshape(is,k)=cff
         enddo
       enddo
#  elif defined EXP_SHAPE
        do is=1,Nsrc                   ! Exponential vertical
          Hs=h(Isrc(is),Jsrc(is))      ! distribution
          cff=5.            ! Hs/z0  (z0 surface layer depth)
          cff1=cff/(1-exp(-cff))
          cff2=0.
          do k=1,N
            Qshape(is,k)=cff1*exp(z_r(Isrc(is),Jsrc(is),k)*cff/Hs)*
     &        (z_w(Isrc(is),Jsrc(is),k)-z_w(Isrc(is),Jsrc(is),k-1))/Hs
            cff2=cff2+Qshape(is,k)
          enddo
          do k=1,N
            Qshape(is,k)=Qshape(is,k)/cff2
          enddo
        enddo
#  elif defined AL_SHAPE
        do is=1,Nsrc                   ! Set-up nondimensional shape
          do k=1,10
            Qshape(is,k)=0.0
          enddo
          do k=11,14
            Qshape(is,k)=0.05
          enddo
          Qshape(is,15)=0.1           ! These most add to unity!
          Qshape(is,16)=0.1           ! These most add to unity!
          Qshape(is,17)=0.1           ! These most add to unity!
          Qshape(is,18)=0.1           ! These most add to unity!
          Qshape(is,19)=0.2
          Qshape(is,20)=0.2
        enddo
#  endif

# elif defined REGIONAL
        do is=1,Nsrc                   ! Exponential vertical
          Hs=h(Isrc(is),Jsrc(is))      ! distribution
          cff=5.            ! Hs/z0  (z0 surface layer depth)
          cff1=cff/(1-exp(-cff))
          cff2=0.
          do k=1,N
            Qshape(is,k)=cff1*exp(z_r(Isrc(is),Jsrc(is),k)*cff/Hs)*
     &        (z_w(Isrc(is),Jsrc(is),k)-z_w(Isrc(is),Jsrc(is),k-1))/Hs
            cff2=cff2+Qshape(is,k)
          enddo
          do k=1,N
            Qshape(is,k)=Qshape(is,k)/cff2
          enddo
        enddo

# else
      ERROR ###  CPP-key 'ANA_PSOURCE' is defined, but no code
      ERROR ###  is provided to set up Dsrc, Isrc, Jsrc, Lsrc.
# endif /* REGIONAL */

      endif   ! iic.eq.ntstart

!
! Set-up vertically integrated mass transport [m3/s] of point
! sources (these may be time-dependent; positive in the positive U-
! or V-direction and vice-versa) and vertically distribute them
! according to mass transport profile chosen above.
!
# if defined RIVER
      ramp=1 !TANH(dt*sec2day*float(iic-ntstart))
      do is=1,Nsrc
        Qbar(is)=ramp*Qbar(is)
      enddo
# elif defined REGIONAL
      ramp=1 !TANH(dt*sec2day*float(iic-ntstart))
      do is=1,Nsrc
        Qbar(is)=ramp*Qbar(is)
      enddo
# else
      ERROR ###  CPP-key 'ANA_PSOURCE' is defined, but no code
      ERROR ###  is provided to set up Qbar(is) analytically.
# endif
      do is=1,Nsrc
        do k=1,N
          Qsrc(is,k)=Qbar(is)*Qshape(is,k)
        enddo
      enddo
!
!  Set-up tracer (tracer units) point Sources/Sinks.
!
# if defined RIVER
      do k=1,N
        do is=1,Nsrc
          Tsrc(is,k,itemp)=Tsrc0(is,itemp)
!          Tsrc(is,k,itemp)=4.+10.*exp(z_r(Isrc(is),Jsrc(is),k)/50.)
          Tsrc(is,k,isalt)=Tsrc0(is,isalt)
#  if defined PASSIVE_TRACER
          Tsrc(is,k,ipas)=Tsrc0(is,itpas)
#  endif
        enddo
      enddo
# elif defined REGIONAL
      do k=1,N
        do is=1,Nsrc
          Tsrc(is,k,itemp)=Tsrc0(is,itemp)
          Tsrc(is,k,isalt)=Tsrc0(is,isalt)
#  if defined PASSIVE_TRACER
          Tsrc(is,k,itpas)=Tsrc0(is,itpas)
#  endif
#  if defined BIOLOGY
          Tsrc(is,k,iNO3_)=Tsrc0(is,iNO3_)
#  endif
        enddo
      enddo
# else
      ERROR ###  CPP-key 'ANA_PSOURCE' is defined, but no code
      ERROR ###  is provided to set up Tsrc(is) analytically.
# endif
      return
      end
#endif /* PSOURCE && ANA_PSOURCE SOLVE3D*/

#ifdef ANA_BRY
!
!---------------------------------------------------------------------
!  Set analytical boundary forcing
!---------------------------------------------------------------------
!
      subroutine ana_bry (tile)
      implicit none
# include "param.h"
      integer tile
# include "compute_tile_bounds.h"
      call ana_bry_tile   (Istr,Iend,Jstr,Jend)
      return
      end
!
      subroutine ana_bry_tile (Istr,Iend,Jstr,Jend) 
      implicit none          
      integer Istr,Iend,Jstr,Jend, i,j,k, itrc
# include "param.h"
# include "boundary.h"
!
# include "compute_auxiliary_bounds.h"
!
# ifdef OBC_WEST
      if (WESTERN_EDGE) then
#  ifdef Z_FRC_BRY
        do j=JstrR,JendR
          zetabry_west(j)=0.
        enddo
#  endif
#  ifdef M2_FRC_BRY
        do j=JstrR,JendR
          ubarbry_west(j)=0.
          vbarbry_west(j)=0.
        enddo
#  endif
#  ifdef SOLVE3D && (defined M3_FRC_BRY || defined T_FRC_BRY)
        do k=1,N
          do j=JstrR,JendR
#   ifdef M3_FRC_BRY
            ubry_west(j,k)=0.
            vbry_west(j,k)=0.
#   endif
#   ifdef T_FRC_BRY
            do itrc=1,NT
              tbry_west(j,k,itrc)=0.
            enddo
#   endif
          enddo
        enddo
#  endif    /* SOLVE3D && (M3_FRC_BRY || T_FRC_BRY)*/
      endif
# endif /* OBC_WEST */
!
# ifdef OBC_EAST
      if (EASTERN_EDGE) then
#  ifdef Z_FRC_BRY
        do j=JstrR,JendR
          zetabry_east(j)=0.
        enddo
#  endif
#  ifdef M2_FRC_BRY
        do j=JstrR,JendR
          ubarbry_east(j)=0.
          vbarbry_east(j)=0.
        enddo
#  endif
#  ifdef SOLVE3D && (defined M3_FRC_BRY || defined T_FRC_BRY)
        do k=1,N
          do j=JstrR,JendR
#   ifdef M3_FRC_BRY
            ubry_east(j,k)=0.
            vbry_east(j,k)=0.
#   endif
#   ifdef T_FRC_BRY
            do itrc=1,NT
              tbry_east(j,k,itrc)=0.
            enddo
#   endif
          enddo
        enddo
#  endif    /* SOLVE3D && (M3_FRC_BRY || T_FRC_BRY)*/
      endif
# endif /* OBC_EAST */
!
# ifdef OBC_SOUTH
      if (SOUTHERN_EDGE) then
#  ifdef Z_FRC_BRY
        do i=IstrR,IendR
          zetabry_south(i)=0.
        enddo
#  endif
#  ifdef M2_FRC_BRY
        do i=IstrR,IendR
          ubarbry_south(i)=0.
          vbarbry_south(i)=0.
        enddo
#  endif
#  ifdef SOLVE3D && (defined M3_FRC_BRY || defined T_FRC_BRY)
        do k=1,N
          do i=IstrR,IendR
#   ifdef M3_FRC_BRY
            ubry_south(i,k)=0.
            vbry_south(i,k)=0.
#   endif
#   ifdef T_FRC_BRY
            do itrc=1,NT
              tbry_south(i,k,itrc)=0.
            enddo
#   endif
          enddo
        enddo
#  endif    /* SOLVE3D && (M3_FRC_BRY || T_FRC_BRY)*/
      endif
# endif /* OBC_SOUTH */
!
# ifdef OBC_NORTH
      if (NORTHERN_EDGE) then
#  ifdef Z_FRC_BRY
        do i=IstrR,IendR
          zetabry_north(i)=0.
        enddo
#  endif
#  ifdef M2_FRC_BRY
        do i=IstrR,IendR
          ubarbry_north(i)=0.
          vbarbry_north(i)=0.
        enddo
#  endif
#  ifdef SOLVE3D && (defined M3_FRC_BRY || defined T_FRC_BRY)
        do k=1,N
          do i=IstrR,IendR
#   ifdef M3_FRC_BRY
            ubry_north(i,k)=0.
            vbry_north(i,k)=0.
#   endif
#   ifdef T_FRC_BRY
            do itrc=1,NT
              tbry_north(i,k,itrc)=0.
            enddo
#   endif
          enddo
        enddo
#  endif    /* SOLVE3D && (M3_FRC_BRY || T_FRC_BRY)*/
      endif
# endif /* OBC_NORTH */
!
      return
      end
#else 
      subroutine empty_analytical
      return
      end
#endif /* defined ANA_BRY */


